function palaeotripspecresolvedfluxes(modeloutput,metadata)
% Write out spectrally resolved model output fluxes into the PALAEOTRIP format. 
% 
% Syntax:
% palaeotripspecresolvedfluxes(modeloutput,metadata)
%
% Inputs: 
%  modeloutput : Structure array containing model output. See below for
%                instructions on how to fill this. 
%  metadata    : Structure array containing user and model run metadata.
%                See below for instructions on how to fill this.
% Example:
%  palaeotripspecresolvedfluxes(modeloutput,metadata)
% 
% Information on structures
%  - If you are unfamiliar with matlab stuctures: these are a really
%  useful and easy to use matlab data type. Have a quick look at the help
%  at http://www.mathworks.com/help/matlab/structures.html
% 
% - modeloutput should contain the following tables (e.g.
% modeloutput.plevel). Each of these (with the exception of plevel, player, and specbins) 
% should be an m by n array where m is the pressure level and n is the spectral range. 
%    Quantities on levels (those marked * are required). 
%    plevel*  : pressure on levels (layer bountaries) (Pa)
%    swspecbins*: edge values of each spectral bin (in mm)
%    lwspecbins*: edge values of each spectral bin (in mm) 
%    Fswdndir : direct solar flux down (W/m/m)
%    Fswdndif : difuse solar flux down (W/m/m)
%    Fswdn    : total solar flux down (Fswdndir+Fswdndif) (W/m/m)
%    Fswup    : solar flux up (W/m/m)
%    Fswnet*  : net solar flux (Fswdn-Fswup) (W/m/m)
%    Flwdn    : thermal flux down (W/m/m)
%    Flwup    : thermal flux up (W/m/m)
%    Flwnet*  : net thermal flux (Flwdn-Flwup) (W/m/m)
%    Quantities on layers (all should be included if any are)
%    player   : pressure at layer centre (Pa)
%    Qsolar   : solar heating rate (K/day)
%    Qtherm   : thermal heating rate (K/day)
%   If the model you are running does not supply data for a field, leave
%   that field empty. 
% 
% - metadata should contain the following fields (e.g. metadata.runcode)
%    runcode*  : String containing the code name of run. See protocol for codes.
%    modelname*: String containing the name (and version number) of model. 
%    username* : String containing your name  (e.g. 'Colin Goldblatt')
%    useremail*: String containing your email (e.g. 'czg@uvic.ca')
%    usernotes*: String containing any notes about this run

Quantlevel = {'Fswdndir','Fswdndif','Fswdn','Fswup','Fswnet','Flwdn','Flwup','Flwnet'};
Quantlayer = {'Qsolar', 'Qtherm'};
Quantreq = {'plevel', 'lwspecbins', 'swspecbins', 'Fswnet', 'Flwnet'};
Quantmeta = {'runcode', 'modelname', 'username', 'useremail', 'usernotes'};
Quanttotal = {'plevel','Fswdndir','Fswdndif','Fswdn','Fswup','Fswnet','Flwdn','Flwup','Flwnet','player', 'Qsolar', 'Qtherm'};
nQuantlevel = length(Quantlevel);
nQuantlayer = length(Quantlayer);
nQuantreq =  length(Quantreq);
nQuantmeta = length(Quantmeta);
nQuanttotal = length(Quanttotal);

% Spectal bins:
nswBins = length(modeloutput.swspecbins);
nlwBins = length(modeloutput.lwspecbins);

% check for required quantities
for ijk = 1:nQuantreq
    if ~isfield(modeloutput, Quantreq{ijk})
        error(['Required quantity ',Quantreq{ijk},' not supplied']);
    end
end

% check whether layers are included
layeroutput = isfield(modeloutput,'player');

% check vectors are all the right way up, flip if not
if modeloutput.plevel(1) > modeloutput.plevel(end)
    for ijk = 1:nQuantlevel
        modeloutput.(Quantlevel{ijk}) = flip(modeloutput.(Quantlevel{ijk}));
    end
end
if layeroutput
    if modeloutput.player(1) > modeloutput.player(end)
        for ijk = 1:nQuantlayer
            modeloutput.(Quantlayer{ijk}) = flip(modeloutput.(Quantlayer{ijk}));
        end
    end
end

% Fill empty fields with null value
nullval = -999.999999;
nullvectorlevel = nullval*ones(size(modeloutput.plevel));
if layeroutput
    nullvectorlayer = nullval*ones(size(modeloutput.player));
end

for ijk = 1:nQuantlevel
    if ~isfield(modeloutput,Quantlevel{ijk})
        modeloutput.(Quantlevel{ijk}) = nullvectorlevel;
    end
end

if layeroutput
    for ijk = 1:nQuantlayer
        if ~isfield(modeloutput,Quantlayer{ijk})
            modeloutput.(Quantlayer{ijk}) = nullvectorlayer;
        end 
    end
end

% Loop through all parameters ands 
for qi = 1:nQuanttotal
    
    % write out the model output
    outputformat = '%12.4f';
    
    if isfield(modeloutput,Quanttotal{qi})
        filenamelayer = [metadata.runcode,'_',Quanttotal{qi},'_layer.txt'];
        filenamelevel = [metadata.runcode,'_',Quanttotal{qi},'_level.txt'];
    end
    
    if isempty(strfind(Quanttotal{qi},'lw'))
        lowerbin = modeloutput.swspecbins(1:(end-1));
        upperbin = modeloutput.swspecbins(2:(end));
    else
        lowerbin = modeloutput.lwspecbins(1:(end-1));
        upperbin = modeloutput.lwspecbins(2:(end));
    end
    
    if any(strcmp(Quantlevel,Quanttotal{qi}))
    fid = fopen(filenamelevel, 'w+');
    % levels
    % fileheader:
    fprintf(fid,'P A L A E O T R I P   M O D E L   O U T P U T   F O R M A T');
    fprintf(fid,'\n');
    fprintf(fid,'palaeotrip.org');
    fprintf(fid,'\n \n');
    % metadata:
    fprintf(fid,['runcode: ', metadata.runcode, '\n']);
    fprintf(fid,['modelname: ', metadata.modelname, '\n']);
    fprintf(fid,['username: ', metadata.username, '\n']);
    fprintf(fid,['useremail: ', metadata.useremail, '\n']);
    fprintf(fid,['usernotes: ', metadata.usernotes, '\n']);
    fprintf(fid,['parameter: ', Quanttotal{qi}, '\n \n']);


    % dataheader
    fprintf(fid,'%12s','P \ binedge');
    for tt=1:length(lowerbin)
        fprintf(fid,'%12s',['[',num2str(lowerbin(tt)),'-',num2str(upperbin(tt)),']um']);
    end
    fprintf(fid,'\n');
    fprintf(fid,'%12s','(Pa)');
    for tt=1:length(lowerbin)
        fprintf(fid,'%12s','(W/m/m)');
    end
    fprintf(fid,'\n');   
    
    % Print model output
    for ii = 1:length(modeloutput.plevel)
        fprintf(fid,'%12.4f',modeloutput.plevel(ii));
        for jj = 1:length(lowerbin)
            fprintf(fid,outputformat,modeloutput.(Quanttotal{qi})(ii,jj));
        end
            fprintf(fid,'\n');
    end
    fclose(fid);

    elseif layeroutput
        fid = fopen(filenamelayer); 
            % layers
            % fileheader:
            fprintf(fid,'P A L A E O T R I P   M O D E L   O U T P U T   F O R M A T');
            fprintf(fid,'\n');
            fprintf(fid,'palaeotrip.org');
            fprintf(fid,'\n \n');

            % metadata:
            fprintf(fid,['runcode: ', metadata.runcode, '\n']);
            fprintf(fid,['modelname: ', metadata.modelname, '\n']);
            fprintf(fid,['username: ', metadata.username, '\n']);
            fprintf(fid,['usernotes: ', metadata.usernotes, '\n']);
            fprintf(fid,['parameter: ', Quanttotal{qi}, '\n \n']);

    % dataheader
    fprintf(fid,'%12s','P \ binedge');
    for tt=1:length(lowerbin)
        fprintf(fid,'%12s',['[',num2str(lowerbin(tt)),'-',num2str(upperbin(tt)),']um']);
    end
    fprintf(fid,'\n');
    fprintf(fid,'%12s','(Pa)');
    for tt=1:length(lowerbin)
        fprintf(fid,'%12s','(K/day)');
    end
    fprintf(fid,'\n'); 

            % Print model output
            for ii = 1:length(modeloutput.plevel)
                fprintf(fid,'%12.4f',modeloutput.plevel(ii));
                for jj = 1:length(lowerbin)
                    fprintf(fid,outputformat,modeloutput.Quanttotal{qi}(ii,jj));
                end
                    fprintf(fid,'\n');
            end
            fclose(fid);
    end
end
end
